/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
  * © 2016 OX Software GmbH, Germany. info@open-xchange.com
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 */

define('io.ox/office/spreadsheet/view/control/formatcategorypicker', [
    'io.ox/office/tk/utils',
    'io.ox/office/tk/control/radiolist',
    'io.ox/office/baseframework/app/appobjectmixin',
    'io.ox/office/spreadsheet/view/labels',
    'gettext!io.ox/office/spreadsheet/main'
], function (Utils, RadioList, AppObjectMixin, Labels, gt) {

    'use strict';

    var // all predefined number format categories to be inserted into the list
        FORMAT_CATEGORIES = ['standard', 'number', 'currency', 'date', 'time', 'datetime', 'percent', 'text', 'scientific', 'fraction'];

    // class FormatCategoryPicker =============================================

    /**
     * A drop-down list control for choosing number format categories, that
     * will load predefined format codes in the format code picker control.
     *
     * @constructor
     *
     * @extends RadioList
     * @extends AppObjectMixin
     *
     * @param {SpreadsheetView} docView
     *  The document view containing this instance.
     *
     * @param {Object} [initOptions]
     *  Optional parameters. Supports all options of the base class RadioList.
     */
    function FormatCategoryPicker(docView, initOptions) {

        // base constructors --------------------------------------------------

        RadioList.call(this, Utils.extendOptions({
            label: _.noI18n('123'),
            tooltip: gt('Number format')
        }, initOptions, {
            updateCaptionMode: 'none'
        }));

        AppObjectMixin.call(this, docView.getApp());

        // initialization -----------------------------------------------------

        this.getNode().addClass('format-category-group');

        // initialize the drop-down list items
        this.waitForImportSuccess(function () {
            // insert all supported list items
            _.each(FORMAT_CATEGORIES, function (category) {
                this.createOptionButton(category, { label: Labels.FORMAT_CATEGORY_LABELS[category] });
            }, this);
        }, this);

        // destroy all class members on destruction
        this.registerDestructor(function () {
            docView = initOptions = null;
        });

    } // class FormatCategoryPicker

    // exports ================================================================

    // derive this class from class RadioList
    return RadioList.extend({ constructor: FormatCategoryPicker });

});
